unit expnoteu;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, Trayicfm, Menus, StdCtrls, Buttons, ShellAPI, ExtCtrls;

type
  TExpNoteForm = class(TTrayIconForm)
    PopupMenu1: TPopupMenu;
    Open1: TMenuItem;
    About1: TMenuItem;
    FindDialog1: TFindDialog;
    Timer1: TTimer;
    BtnFind: TSpeedButton;
    BtnPrune: TSpeedButton;
    BtnHelp: TSpeedButton;
    BtnAbout: TSpeedButton;
    Image1: TImage;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormShow(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure BtnPruneClick(Sender: TObject);
    procedure BtnAboutClick(Sender: TObject);
    procedure BtnHelpClick(Sender: TObject);
    procedure BtnFindNoteClick(Sender: TObject);
    procedure FindDialog1Find(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    { Private declarations }
    holdMap: THandle;
    ReallyClose : Boolean;
    fNoteToFind : String;
    fFoundItem  : String;
    fFoundNote  : String;
    procedure wmSyscommand(VAR Msg: TWMSysCommand);
      message WM_SYSCOMMAND;
    procedure wmEndSession(VAR Msg: TWMEndSession);
      message WM_ENDSESSION;
    procedure wmQueryEndSession(VAR Msg: TWMQueryEndSession);
      message WM_QUERYENDSESSION;
  public
    { Public declarations }
  end;

var
  ExpNoteForm: TExpNoteForm;

implementation
uses commctrl, mmsystem, Registry, FileCtrl, aboutbox;

const // duplicated in DLLSHARE.INC
  expKey = '\Software\PC Magazine\Explorer Notes\1.0\notes\';


{$R *.DFM}

function ExpNote_Activate : THandle; Export;
  Far; StdCall; External 'EN_DLL.DLL' name 'ExpNote_Activate';

PROCEDURE ExpNote_DeActivate(holdMap : Integer); Export;
  Far; StdCall; External 'EN_DLL.DLL' name 'ExpNote_DeActivate';

procedure TExpNoteForm.FormCreate(Sender: TObject);
VAR Buffer : ARRAY[0..15] OF Byte;
begin
  inherited;
  ClientWidth := BtnAbout.Left + BtnAbout.Width+1;
  ClientHeight := BtnAbout.Top + BtnAbout.Height+1;
  Application.HelpFile := ChangeFileExt(Application.ExeName, '.HLP');
  SetupTrayIcon(1021, Application.Icon.Handle, 'Explorer Notes');
  HoldMap := ExpNote_Activate;
  WITH TRegistry.Create DO
    try
      OpenKey('Software\Microsoft\Windows\CurrentVersion\Explorer',
        False);
      ReadBinaryData('ShellState', Buffer, 16);
      // By observation, this bit indicates the state of the
      // Explorer View Option "Hide MS-DOS File Extension..."
      IF Buffer[4] AND 2 <> 2 THEN
        WinHelp(Handle, PChar(Application.HelpFile+'>Warning'),
          HELP_CONTEXT, 11)
      ELSE
        begin
          OpenKey('CabinetState', False);
          ReadBinaryData('Settings', Buffer, 12);
          // By observation, this bit indicates the state of the
          // Explorer View Option "Display the full MS-DOS path..."
          IF Buffer[4] AND 1 <> 1 THEN
            WinHelp(Handle, PChar(Application.HelpFile+'>Warning'),
              HELP_CONTEXT, 11);
        end;
    finally
      free;
    end;
end;

procedure TExpNoteForm.FormDestroy(Sender: TObject);
begin
  inherited;
  ExpNote_DeActivate(HoldMap);
end;

procedure TExpNoteForm.wmSyscommand(VAR Msg: TWMSysCommand);
begin
  // Don't terminate on close; just minimize
  IF Msg.CmdType AND $FFF0 = SC_MINIMIZE THEN
    BEGIN
      Close;
      Msg.Result := 0;
    END
  ELSE Inherited;
end;

procedure TExpNoteForm.wmQueryEndSession(VAR Msg:
  TWMQueryEndSession);
// Register so we get notified of ACTUAL end-session
begin
  Inherited;
  Msg.Result := 1;
end;

procedure TExpNoteForm.wmEndSession(VAR Msg: TWMEndSession);
// Allow shutdown if Windows is shutting down
begin
  Inherited;
  IF Msg.EndSession THEN
    begin
      ReallyClose := True;
      Close;
    end;
  Msg.Result := 0;
end;

procedure TExpNoteForm.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  inherited;
  CanClose := ReallyClose;
  IF NOT ReallyClose THEN
    BEGIN
      PlaySound('RestoreDown', 0, SND_ALIAS OR SND_ASYNC);
      Hide;
    END;
end;

procedure TExpNoteForm.FormShow(Sender: TObject);
VAR tabd : TAppBarData;
begin
  FillChar(tabd, SizeOf(tabd), 0);
  PlaySound('RestoreUp', 0, SND_ALIAS OR SND_ASYNC);
  tabd.cbSize := SizeOf(tabd);
  IF SHAppBarMessage(ABM_GETTASKBARPOS, Tabd)=0 THEN Exit;
  // only the .rc field is valid, and only one of its fields
  // will be inside the screen dimensions; that field defines
  // which edge the appbar is docked to
  IF tabd.rc.top > 0 THEN
    begin // bottom edge
      Left := Screen.Width - Width;
      Top  := tabd.rc.top - Height;
    end
  ELSE IF tabd.rc.bottom < Screen.Height THEN
    begin // top edge
      Left := Screen.Width - Width;
      Top  := tabd.rc.bottom;
    end
  ELSE IF tabd.rc.left > 0 THEN
    begin // right edge
      Left := tabd.rc.left - Width;
      Top  := Screen.Height - Height;
    end
  ELSE IF tabd.rc.right < Screen.Width THEN
    begin // left edge
      Left := tabd.rc.right;
      Top  := Screen.Height - Height;
    end;
end;

procedure TExpNoteForm.Open1Click(Sender: TObject);
begin
  Show;
end;

procedure TExpNoteForm.BtnPruneClick(Sender: TObject);
VAR
  LogFileName : String;
  LogFile     : TextFile;
  WriteOK     : Boolean;

  function OkKey(const key : String; level : Integer) : Boolean;
  VAR
    TSL : TStringList;
    N   : Integer;
    S   : String;
  begin
    WITH TRegistry.Create DO
      try
        IF OpenKey(expKey+key, False) THEN
          begin
            IF (level=1) AND
               (GetDriveType(PChar(Key+'\')) <> DRIVE_FIXED) THEN
              begin
                WriteLn(LogFile, '*** Drive '+ key +'\ not a fixed '+
                  'disk; not checked for orphaned notes');
                Result := True;
                Exit;
              end;
            // If the key is non-empty and doesn't represent
            // an existing file or directory, it's NOT OK!
            IF (key<>'') AND (NOT FileExists(key)) AND
              (NOT DirectoryExists(key)) THEN
              begin
                S := ReadString('');
                IF (S <> '') AND WriteOK THEN
                  try
                    WriteLn(LogFile, 'File: ', key);
                    WriteLn(LogFile, 'Note: ', S);
                  except
                    WriteOK := False;
                  end;
                Result := FALSE;
                Exit;
              end;
            TSL := TStringList.Create;
            try
              GetKeyNames(TSL);
              IF key='' THEN S := ''
              ELSE S := key + '\';
              FOR N := TSL.Count-1 DOWNTO 0 DO
                begin
                  IF NOT OkKey(S+TSL[N], Level+1) THEN
                    begin
                      DeleteKey(expKey+S+TSL[N]);
                      TSL.Delete(N);
                    end;
                end;
              Result := (TSL.Count>0) OR (ReadString('')<>'');
            finally
              TSL.free;
            end;
          end
        ELSE Result := False;
      finally
        free;
      end;
  end;
begin
  LogFileName := ChangeFileExt(Application.ExeName, '.LOG');
  AssignFile(LogFile, LogFileName);
  WriteOK := True;
  try
    Rewrite(LogFile);
    WriteLn(LogFile, '*** Explorer Notes, copyright (c) 1996 by ',
      'Ziff-Davis Publishing');
    WriteLn(LogFile, '*** LOG of orphaned notes deleted ',
      FormatDateTime('dddddd, tt', Now));
  except WriteOK := False; end;
  OkKey('', 0);
  IF WriteOK THEN
    begin
      CloseFile(LogFile);
      ShowMessage('Orphaned notes logged to'#13+LogFileName);
    end
  else ShowMessage('Logging of orphaned notes failed');
end;

procedure TExpNoteForm.BtnAboutClick(Sender: TObject);
begin
  WITH TAboutForm.Create(Application) DO
    try
      ShowModal;
    finally
      free;
    end;
end;

procedure TExpNoteForm.BtnHelpClick(Sender: TObject);
begin
  IF ((GetAsyncKeyState(VK_SHIFT) AND $8000) <> 0) AND
     ((GetAsyncKeyState(VK_CONTROL) AND $8000) <> 0) THEN
    BEGIN
      ReallyClose := True;
      Close;
    END
  ELSE Application.HelpCommand(HELP_FINDER, 0);
end;

procedure TExpNoteForm.BtnFindNoteClick(Sender: TObject);
begin
  fFoundItem := '';
  WITH FindDialog1 DO
    begin
      Timer1.Enabled := True;
      Execute;
    end;
end;

procedure TExpNoteForm.Timer1Timer(Sender: TObject);
begin
  IF FindDialog1.Handle = 0 THEN Exit;
  Timer1.Enabled := False;
  SetWindowPos(FindDialog1.Handle, HWND_TOPMOST, 0, 0, 0, 0,
    SWP_NOSIZE OR SWP_NOMOVE);
  SetWindowText(FindDialog1.Handle, 'Find Files w/ Notes '+
    'Containing...');
end;

procedure TExpNoteForm.FindDialog1Find(Sender: TObject);
VAR FreshSearch : Boolean;
  function CheckOneKey(const key : String) : Boolean;
  VAR
    TSL   : TStringList;
    N     : Integer;
    KeyVal,
    nuKey : String;
  begin
    Result := False;
    WITH TRegistry.Create DO
      try
        IF OpenKey(expKey+key, False) THEN
          begin
            IF fFoundItem='' THEN
              begin
                IF frMatchCase IN FindDialog1.Options THEN
                  KeyVal := ReadString('')
                ELSE KeyVal := UpperCase(ReadString(''));
                IF Pos(fNoteToFind, KeyVal) > 0 THEN
                  BEGIN
                    Result := True;
                    fFoundItem := Key;
                    fFoundNote := ReadString('');
                    Exit;
                  END;
              end;
            IF (fFoundItem<>'') AND (key=fFoundItem) THEN
              fFoundItem := '';
            TSL := TStringList.Create;
            try
              GetKeyNames(TSL);
              IF key='' THEN nuKey := ''
              ELSE nuKey := key + '\';
              FOR N := 0 TO TSL.Count-1 DO
                begin
                  IF CheckOneKey(nuKey+TSL[N]) THEN
                    begin
                      Result := True;
                      Exit;
                    end;
                end;
            finally
              TSL.free;
            end;
          end;
      finally
        free;
      end;
  end;
begin
  FreshSearch := fFoundItem = '';
  IF frMatchCase IN FindDialog1.Options THEN
    fNoteToFind := FindDialog1.FindText
  ELSE fNoteToFind := UpperCase(FindDialog1.FindText);
  IF fNoteToFind = '' THEN Exit;
  IF CheckOneKey('') THEN
    begin
      IF FileExists(fFoundItem) OR DirectoryExists(fFoundItem) THEN
        ShellExecute(Handle, NIL, 'EXPLORER.EXE',
          PChar(Format('/select,%s',[fFoundItem])), nil, SW_RESTORE)
      ELSE
        ShowMessage(Format('The note "%s" contains "%s", but '+
          'file/folder %s does not exist. Try Find Next.',
          [fFoundNote, FindDialog1.FindText, fFoundItem]));
    end
  ELSE IF FreshSearch THEN
    ShowMessage(Format('No notes contain "%s"',
      [FindDialog1.FindText]))
  ELSE ShowMessage(Format('No more notes contain "%s"',
      [FindDialog1.FindText]))
end;


end.

