unit expsubcl;
interface
USES Windows;

procedure CBT_DeActivate(holdMap: THandle); Export; Stdcall;
function CBT_Activate : THandle; Export; StdCall;

implementation
USES Ole2, CommCtrl, Messages, SysUtils, Registry;
{$I DLLSHARE.INC}

// FIGNU07.TXT part 1 begins here
VAR
  // These variables are "local" to the current instance of the DLL
  loc_OldCBTH  : HHook;
  loc_SHOldP   : Integer;
  loc_LVH      : HWnd;
  loc_lvBuffer : ARRAY[0..MAX_PATH] OF Char;

function GetOldCBTH : HHook;
// If current DLL instance does not have old CBT Hook, get it
// from memory-mapped file; otherwise, return existing value
VAR
  hMap  : THandle;
  PHook : ^HHook;
begin
  IF loc_OldCBTH = 0 THEN
    begin
      hMap := CreateFileMapping(-1, NIL, PAGE_READWRITE, 0,
        SizeOf(HHook), MapName);
      PHook := MapViewOfFile(hMap, FILE_MAP_ALL_ACCESS, 0, 0, 0);
      loc_OldCBTH := PHook^;
      UnMapViewOfFile(PHook);
      CloseHandle(hMap);
    end;
  Result := loc_OldCBTH;
end;
// FIGNU07.TXT part 1 ends here

function WSHproc(HWnd: Integer; Msg, wparam: Integer; lparam:
  PLVDispInfo) : Integer; Export; StdCall;
  // Subclass window procedure for parent of SysListView32 in
  // Explorer - handles LVN_GETDISPINFO notification requesting
  // text for the Notes column by pulling the note from the Registry
VAR
  vPathBuffer : ARRAY[0..MAX_PATH] OF Char;
  vNote       : String;

  Function ItemString : PChar;
  begin
    ListView_GetItemText(lParam.hdr.hwndFrom,
      lParam.Item.iItem, 0, @loc_lvBuffer, MAX_PATH);
    Result := @loc_lvBuffer;
  end;
begin
  Result := 0;
  // IF it's a LVN_GETDISPINFO notification requesting text
  // for item 4, give it some text!
  IF (Msg=WM_NOTIFY) AND
     (lParam.hdr.Code=LVN_GETDISPINFO) AND
     (lParam.Item.iSubItem=4) AND
     (lParam.Item.mask AND LVIF_TEXT > 0) THEN
    begin
      // This only works if parent is Explorer, NOT dialog
      GetWindowText(GetParent(HWnd), vPathBuffer, MAX_PATH);
      IF StrLen(vPathBuffer) = 0 THEN Exit;
      IF StrLComp(vPathBuffer, 'Exploring - ', 12) = 0 THEN
        StrCopy(vPathBuffer, vPathBuffer+12);
      IF vPathBuffer[StrLen(vPathBuffer)-1] <> '\' THEN
        StrCat(vPathBuffer, '\');
      StrLCat(vPathBuffer, ItemString, MAX_PATH);
      // vPathBuffer now contains full pathname of file
      WITH TRegistry.Create DO
        try
          OpenKey(expKey + StrPas(vPathBuffer), False);
          vNote := ReadString('');
          IF vNote <> '' THEN StrLCopy(lParam.Item.pszText,
            PChar(vNote), lParam.Item.cchTextMax);
        finally
          free;
        end;
    end
  ELSE IF (Msg=WM_NOTIFY) AND (lParam.hdr.Code=LVN_COLUMNCLICK) AND
    (PNMListView(lParam).iSubItem=4) THEN MessageBeep(0)
  ELSE Result := CallWindowProc(Pointer(loc_SHOldP), HWnd, Msg,
    wParam, Integer(lParam));
end;

function WSHprocD(HWnd: Integer; Msg, wparam: Integer; lparam:
  PLVDispInfo) : Integer; Export; StdCall;
  // Subclass window procedure for parent of SysListView32 in
  // file dialog - handles LVN_GETDISPINFO notification requesting
  // text for the Notes column by pulling the note from the Registry
VAR
  vPathBuffer : ARRAY[0..MAX_PATH] OF Char;
  vNote       : String;

  Function ItemString : PChar;
  begin
    ListView_GetItemText(lParam.hdr.hwndFrom,
      lParam.Item.iItem, 0, @loc_lvBuffer, MAX_PATH);
    Result := @loc_lvBuffer;
  end;
begin
  Result := 0;
  // IF it's a LVN_GETDISPINFO notification requesting text
  // for item 4, give it some text!
  IF (Msg=WM_NOTIFY) AND
     (lParam.hdr.Code=LVN_GETDISPINFO) AND
     (lParam.Item.iSubItem=4) AND
     (lParam.Item.mask AND LVIF_TEXT > 0) THEN
    begin
      GetCurrentDirectory(MAX_PATH, vPathBuffer);
      IF vPathBuffer[StrLen(vPathBuffer)-1] <> '\' THEN
        StrCat(vPathBuffer, '\');
      StrLCat(vPathBuffer, ItemString, MAX_PATH);
      // vPathBuffer now contains full pathname of file
      WITH TRegistry.Create DO
        try
          OpenKey(expKey + StrPas(vPathBuffer), False);
          vNote := ReadString('');
          IF vNote <> '' THEN StrLCopy(lParam.Item.pszText,
            PChar(vNote), lParam.Item.cchTextMax);
        finally
          free;
        end;
    end
  ELSE IF (Msg=WM_NOTIFY) AND (lParam.hdr.Code=LVN_COLUMNCLICK) AND
    (PNMListView(lParam).iSubItem=4) THEN MessageBeep(0)
  ELSE Result := CallWindowProc(Pointer(loc_SHOldP), HWnd, Msg,
    wParam, Integer(lParam));
end;

procedure AddNotesColumn(LVH : HWnd);
VAR
  LVC         : TLVColumn;
  LVI         : TLVItem;
  LVCBuff     : ARRAY[0..MAX_PATH] OF Char;
  NumItems, N : Integer;
begin
  LVC.Mask       := LVCF_Text;
  LVC.cchTextMax := MAX_PATH;
  LVC.pszText    := @LVCBuff;
  StrCopy(LVC.pszText, '(none)');
  SendMessage(LVH, LVM_GETCOLUMN, 4, Integer(@LVC));
  // If Notes column was not present, add it
  IF StrComp(LVC.pszText, 'Notes') <> 0 THEN
    BEGIN
      LVC.mask       := LVCF_FMT OR LVCF_SUBITEM OR LVCF_TEXT
                        OR LVCF_WIDTH;
      LVC.Fmt        := 0;
      LVC.Cx         := 200;
      LVC.cchTextMax := MAX_PATH;
      LVC.iSubItem   := 4;
      LVC.pszText    := @LVCBuff;
      StrCopy(LVC.pszText, 'Notes');
      SendMessage(LVH, LVM_INSERTCOLUMN, 4, Integer(@LVC));
    END;
  // For each item, set the new column text to LPSTR_TEXTCALLBACK.
  NumItems := SendMessage(LVH, LVM_GETITEMCOUNT, 0, 0);
  FOR N := 0 TO NumItems-1 DO
    BEGIN
      FillChar(LVI, SizeOf(LVI), 0);
      LVI.mask     := LVIF_TEXT;
      LVI.iItem    := N;
      LVI.iSubItem := 4;
      LVI.pszText  := LPSTR_TEXTCALLBACK;
      SendMessage(LVH, LVM_SETITEM, 4, Integer(@LVI));
    END;
end;

// FIGNU07.TXT part 2 begins here
function CBTProc(code, wparam, lParam : Integer) : Integer;
  Export; StdCall;
VAR hPrnt : HWnd;
type vWinType = (vwt_Igno, vwt_List, vwt_ShDv, vwt_Expl, vwt_FilD);

  function WhatType(H : HWnd) : vWinType;
  VAR Buffer : ARRAY[0..255] OF Char;
  begin
    Result := vwt_Igno;
    GetClassName(H, Buffer, 255);
    IF (StrComp(Buffer, 'SysListView32')=0) THEN
      Result := vwt_List  // file listview
    ELSE IF (StrComp(Buffer, 'SHELLDLL_DefView')=0) THEN
      Result := vwt_ShDv  // parent of file listview
    ELSE IF (StrComp(Buffer, 'ExploreWClass')=0) THEN
      Result := vwt_Expl  // 2-pane "explore" view
    ELSE IF (StrComp(Buffer, 'CabinetWClass')=0) THEN
      Result := vwt_Expl  // 1-pane "open" view
    ELSE IF (StrComp(Buffer, '#32770')=0) AND
            (GetWindowTextLength(H) > 0) AND
            (GetWindowLong(H, GWL_STYLE) AND
             (WS_POPUP OR DS_MODALFRAME) <> 0) THEN
      Result := vwt_FilD; //  system file dialog
  end;

begin
  CASE Code OF
    HCBT_CREATEWND : IF WhatType(wParam) = vwt_List THEN
      // if a list view was created...
      begin
        hPrnt := PCBTCreateWnd(lParam)^.lpcs^.HWndParent;
        // if its parent is a SHELLDLL_DefView...
        IF WhatType(hPrnt) = vwt_ShDv THEN
          // if the parent needs to be subclassed...
          IF (loc_SHOldP=0) OR (loc_SHOldP = GetWindowLong(hPrnt,
            GWL_WNDPROC)) THEN
            CASE WhatType(GetParent(hPrnt)) OF
              vwt_Expl : begin
                loc_LVH := wParam;
                loc_SHOldP := SetWindowLong(hPrnt,
                  GWL_WNDPROC, Integer(@WSHProc));
              end;
              vwt_FilD  : begin
                loc_LVH := wParam;
                loc_SHOldP := SetWindowLong(hPrnt,
                  GWL_WNDPROC, Integer(@WSHProcD));
              end;
            END;
      end;
    HCBT_SETFOCUS : IF WhatType(wParam) = vwt_List THEN
      // if a list view got the focus...
      CASE WhatType(GetParent(GetParent(wParam))) OF
        vwt_Expl,
        vwt_FilD : AddNotesColumn(wParam);
      END;
    HCBT_DESTROYWND : IF (WhatType(wParam) = vwt_ShDv) AND
      // if a SHELLDLL_DefView was destroyed...
       (WhatType(GetParent(wParam)) <> vwt_Igno) AND
       (loc_LVH <> 0) THEN
      begin
        AddNotesColumn(loc_LVH);
        loc_LVH := 0;
      end;
  END;
  Result := CallNextHookEx(GetOldCBTH, code, wParam, lParam);
end;

function CBT_Activate : THandle; Export; StdCall;
VAR
  hMap   : THandle;
  PHook  : ^HHook;
  buffer : ARRAY[0..MAX_PATH] OF Char;
begin
  // create memory-mapped file to hold old CBT handle; let
  // ExpNotes keep handle of mapping
  Result := CreateFileMapping(-1, NIL, PAGE_READWRITE, 0,
    SizeOf(HHook), MapName);
  hMap := CreateFileMapping(-1, NIL, PAGE_READWRITE, 0,
    SizeOf(HHook), MapName);
  PHook := MapViewOfFile(hMap, FILE_MAP_ALL_ACCESS, 0, 0, 0);
  PHook^ := SetWindowsHookEx(WH_CBT, @CBTProc, hInstance,0);
  UnMapViewOfFile(PHook);
  CloseHandle(hMap);
end;

PROCEDURE CBT_DeActivate(holdMap: THandle); Export; Stdcall;
begin
  UnhookWindowsHookEx(GetOldCBTH);
  CloseHandle(holdMap);
end;
// FIGNU07.TXT part 2 ends here

end.
