unit exshellu;
interface
uses Ole2, Windows, SysUtils, Classes, ShellObj;
{$I DLLSHARE.INC}
// FIGNU04.TXT part 1 begins here
VAR DllRefCount, LockCount : Integer;

type
  IMyShellExtension = class;

  ISatelliteUnknown = class(IUnknown)
  // Meant to be contained by an IMyShellExtension.
  // Reflects the three IUnknown functions to its container.
  private
    fContainer : IMyShellExtension;
  public
    constructor Create(vContainer: IMyShellExtension);
    function QueryInterface(const WantIID: TIID;
      var ReturnedObject): HResult; override;
    function AddRef: Longint; override;
    function Release: Longint; override;
  end;

  IMyShellExtInit = class(ISatelliteUnknown)
  public
    function Initialize(pidlFolder:PItemIDList; lpdobj: IDataObject;
      hKeyProgID:HKEY):HResult; virtual; stdcall;
  end;

  IMyCopyHook = class(ISatelliteUnknown)
  public
    function CopyCallback(Wnd:HWND;wFunc:UINT; wFlags:UINT;
      pszSrcFile:LPSTR; dwSrcAttribs:DWORD; pszDestFile:LPSTR;
      dwDestAttribs:DWORD):UINT; virtual; stdcall;
  end;

  IMyContextMenu = class(ISatelliteUnknown)
  public
    function QueryContextMenu(Menu:HMENU; indexMenu:UINT;
      idCmdFirst:UINT;idCmdLast:UINT; uFlags:UINT):HResult;
      virtual; stdcall;
    function InvokeCommand(lpici: PCMINVOKECOMMANDINFO): HResult;
      virtual; stdcall;
    function GetCommandString(idCmd:UINT; uType:UINT; var pwReserved:
      UINT; pszName:LPSTR; cchMax:UINT):HResult; virtual; stdcall;
  end;

  IMyShellExtension = class(IUnknown)
  private
    FRefCount     : Integer;
    FShellExtInit : IMyShellExtInit; // Satellite interface
    FCopyHook     : IMyCopyHook;     // Satellite interface
    FContextMenu  : IMyContextMenu;  // Satellite interface
    FNumFiles     : Integer;
    FInitFiles    : TStringList;
  public
    constructor Create;
    destructor Destroy; override;
    {IUnknown functions}
    function QueryInterface(const WantIID: TIID;
      var ReturnedObject): HResult; override;
    function AddRef: LongInt; override;
    function Release: LongInt; override;
  end;
// FIGNU04.TXT part 1 ends here

  IMyClassFactory = Class(IClassFactory)
  private
    FRefCount : Integer;
  public
    constructor Create;
    destructor Destroy; override;
    function QueryInterface(const WantIID: TIID;
      var ReturnedObject): HResult; override;
    function AddRef: LongInt; override;
    function Release: LongInt; override;
    function CreateInstance(unkOuter: IUnknown; const WantIid: TIID;
      var ReturnedObject): HResult; override;
    function LockServer(fLock: BOOL): HResult; override;
  end;

implementation
uses ShellApi, ApiDlgU, Registry;

(****** IMyShellExtension ******)
// FIGNU04.TXT part 2 begins here
constructor IMyShellExtension.Create;
begin
  Inherited Create;
  FRefCount     := 0;
  FInitFiles    := TStringList.Create;
  FShellExtInit := IMyShellExtInit.Create(Self);
  FCopyHook     := IMyCopyHook.Create(Self);
  FContextMenu  := IMyContextMenu.Create(Self);
  Inc(DllRefCount);
end;

destructor IMyShellExtension.Destroy;
begin
  Dec(DllRefCount);
  FShellExtInit.Free;
  FCopyHook.    Free;
  FContextMenu. Free;
  FInitFiles.   Free;
  Inherited Destroy;
end;

function IMyShellExtension.QueryInterface(const WantIID: TIID;
  var ReturnedObject): HResult;
VAR P : IUnknown;
begin
  IF IsEqualIID(WantIID, IID_IUnknown) THEN
    P := FContextMenu
  ELSE IF IsEqualIID(WantIID, IID_IContextMenu) THEN
    P := FContextMenu
  ELSE IF IsEqualIID(WantIID, IID_IShellextInit) THEN
    P := FShellExtInit
  ELSE IF IsEqualIID(WantIID, IID_IShellCopyHook) THEN
    P := FCopyHook
  ELSE P := nil;
  Pointer(ReturnedObject) := P;
  IF P = NIL THEN Result := E_NOINTERFACE
  ELSE
    begin
      P.AddRef;
      Result := S_OK;
    end;
end;

function IMyShellExtension.AddRef: LongInt;
begin
  Inc(FRefCount);
  Result := FRefCount;
end;

function IMyShellExtension.Release: LongInt;
begin
  Dec(FRefCount);
  Result := FRefCount;
  IF FRefCount = 0 THEN Free;
end;
// FIGNU04.TXT part 2 ends here

(****** ISatelliteUnknown ******)
constructor ISatelliteUnknown.Create(vContainer: IMyShellExtension);
begin
  fContainer := vContainer;
end;

function ISatelliteUnknown.QueryInterface(const WantIID: TIID;
  var ReturnedObject): HResult;
begin
  Result := fContainer.QueryInterface(WantIid, ReturnedObject);
end;

function ISatelliteUnknown.AddRef: LongInt;
begin
  Result := fContainer.AddRef;
end;

function ISatelliteUnknown.Release: LongInt;
begin
  Result := fContainer.Release;
end;

(****** IMyShellExtInit ******)
function IMyShellExtInit.Initialize(pidlFolder:PItemIDList;
  lpdobj: IDataObject; hKeyProgID:HKEY):HResult;
VAR
  buffer  : ARRAY[0..MAX_PATH] OF Char;
  Medium  : TStgMedium;
  fe      : TFormatEtc;
  N       : Integer;
begin
  IF lpdobj = NIL THEN
    begin
      result := E_FAIL;
      Exit;
    end;
  WITH fe DO
    begin
      cfFormat := CF_HDROP;
      ptd      := NIL;
      dwAspect := DVASPECT_CONTENT;
      lindex   := -1;
      tymed    := TYMED_HGLOBAL;
    end;
  Result := lpdobj.GetData(fe, medium);
  IF failed(Result) THEN Exit;
  fContainer.FNumFiles :=
    DragQueryFile(medium.hGlobal, -1, NIL, 0);
  fContainer.FInitFiles.Clear;
  IF fContainer.FNumFiles > 0 THEN
    begin
      FOR N := 0 TO fContainer.FNumFiles-1 DO
        begin
          DragQueryFile(medium.hGlobal, N, Buffer, 260);
          fContainer.FInitFiles.Add(StrPas(Buffer));
        end;
      Result := NOERROR;
    end
  ELSE Result := E_FAIL;
  ReleaseStgMedium(medium);
end;

(****** IMyCopyHook ******)
function IMyCopyHook.CopyCallback(Wnd:HWND;wFunc:UINT; wFlags:UINT;
  pszSrcFile:LPSTR; dwSrcAttribs:DWORD; pszDestFile:LPSTR;
  dwDestAttribs:DWORD):UINT;
begin
  Result := IDYES;
  CASE wFunc OF
    FO_COPY   : ; // do nothing on copy
    FO_MOVE,
    FO_RENAME : WITH TRegistry.Create DO
        try
          MoveKey(expKey + StrPas(pszSrcFile),
                  expKey + StrPas(pszDestFile), True);
        finally
          free;
        end;
    FO_DELETE : WITH TRegistry.Create DO
      try
        DeleteKey(expKey + StrPas(pszSrcFile));
      finally
        free;
      end;
  end;
end;

(****** IMyContextMenu ******)
const IDOFFSET_ADDEDIT = 0;

function IMyContextMenu.QueryContextMenu(Menu:HMENU; indexMenu:
  UINT; idCmdFirst:UINT;idCmdLast:UINT; uFlags:UINT):HResult;
begin
  IF fContainer.FNumFiles > 1 THEN
    InsertMenu(Menu, IndexMenu, MF_STRING OR MF_BYPOSITION,
      idCmdFirst + IDOFFSET_ADDEDIT, '&Append note...')
  ELSE InsertMenu(Menu, IndexMenu, MF_STRING OR MF_BYPOSITION,
    idCmdFirst + IDOFFSET_ADDEDIT, '&Add/Edit note...');
  Result := MakeResult(SEVERITY_SUCCESS, 0, IDOFFSET_ADDEDIT+1);
end;

function IMyContextMenu.InvokeCommand(lpici: PCMINVOKECOMMANDINFO):
  HResult;
VAR
  S, WasS : String;
  N       : Integer;
begin
  Result := E_FAIL;
  IF HiWord(Integer(lpici.lpVerb)) <> 0 THEN Exit;
  IF LoWord(Integer(lpici.lpVerb)) > IDOFFSET_ADDEDIT THEN
    begin
      result := E_INVALIDARG;
      exit;
    end;
  WITH TRegistry.Create DO
    try
      IF fContainer.FNumFiles = 1 THEN // add/edit note for one file
        begin
          IF OpenKey(expKey+fContainer.FInitFiles[0], False) THEN
            S := ReadString('')
          ELSE S := '';
          IF EdNote(fContainer.FInitFiles[0], S) THEN
            begin
              IF S <> '' THEN // write the new note
                begin
                  OpenKey(expKey+fContainer.FInitFiles[0], True);
                  WriteString('', S);
                end
              else
                begin // delete the note and perform any cleanup
                  S := fContainer.FInitFiles[0];
                  IF DeleteKey(expKey+fContainer.FInitFiles[0]) THEN
                    WHILE True DO
                      begin
                        S := ExtractFileDir(S);
                        IF S[Length(S)] = '\' THEN
                          SetLength(S, Length(S)-1);
                        IF NOT OpenKey(expKey + S, False) THEN Break;
                        IF (ReadString('')<>'') OR HasSubKeys THEN
                          Break;
                        IF NOT DeleteKey(expKey + S) THEN Break;
                      end;
                end;
              SHChangeNotify(SHCNE_UPDATEITEM, SHCNF_FLUSHNOWAIT OR
                SHCNF_PATH, Pchar(fContainer.FInitFiles[0]), NIL);
            end;
        end
      else // append note to many files
        begin
          S := '';
          IF NOT EdNote('', S) THEN Exit;
          IF S = '' THEN Exit;
          FOR N := 0 TO fContainer.FNumFiles-1 DO
            begin
              IF OpenKey(expKey+fContainer.FInitFiles[N], False) THEN
                WasS := ReadString('')
              ELSE WasS := '';
              IF WasS = '' THEN WasS := S
              ELSE WasS := WasS + '; ' + S;
              IF Length(WasS) > MAX_PATH THEN
                SetLength(WasS, MAX_PATH);
              OpenKey(expKey+fContainer.FInitFiles[N], True);
              WriteString('', WasS);
              SHChangeNotify(SHCNE_UPDATEITEM, SHCNF_FLUSHNOWAIT OR
                SHCNF_PATH, Pchar(fContainer.FInitFiles[N]), NIL);
            end;
          fContainer.FInitFiles.Clear;
        end;
    finally
      free;
    end;
  Result := NOERROR;
end;

function IMyContextMenu.GetCommandString(idCmd:UINT; uType:UINT;
  var pwReserved: UINT; pszName:LPSTR; cchMax:UINT):HResult;
begin
  Result := NOERROR;
  IF idCmd > IDOFFSET_ADDEDIT then Result := E_INVALIDARG
  ELSE IF fContainer.FNumFiles > 1 THEN
    CASE uType OF
      GCS_VERB     : StrLCopy(pszname, 'AppendNote', cchMax);
      GCS_HELPTEXT : StrLCopy(pszName, 'Append note for multiple'+
        ' files using PC Magazine utility ExpNotes', cchMax);
    end
  ELSE
    CASE uType OF
      GCS_VERB     : StrLCopy(pszname, 'AddEditNote', cchMax);
      GCS_HELPTEXT : StrLCopy(pszName, 'Add or edit notes for '+
        'display by the PC Magazine utility ExpNotes', cchMax);
    end;
end;

(****** IMyClassFactory ******)
constructor IMyClassFactory.Create;
begin
  Inherited Create;
  Inc(DllRefCount);
  FRefCount := 0;
end;

destructor IMyClassFactory.Destroy;
begin
  Dec(DllRefCount);
  Inherited Destroy;
end;

function IMyClassFactory.QueryInterface(const WantIID: TIID;
  var ReturnedObject): HResult;
begin
  IF IsEqualIID(WantIid, IID_IUnknown) OR
     IsEqualIID(WantIid, IID_IClassFactory) THEN
    begin
      Pointer(ReturnedObject) := Self;
      AddRef;
      Result := S_OK;
    end
  ELSE
    begin
      Pointer(ReturnedObject) := NIL;
      Result := E_NOINTERFACE;
    end;
end;

function IMyClassFactory.AddRef: LongInt;
begin
  Inc(FRefCount);
  Result := FRefCount;
end;

function IMyClassFactory.Release: LongInt;
begin
  Dec(FRefCount);
  Result := FRefCount;
  IF FRefCount = 0 THEN Free;
end;

function IMyClassFactory.CreateInstance(unkOuter: IUnknown;
  const WantIid: TIID; var ReturnedObject): HResult;
VAR MyShellExtension: IMyShellExtension;
begin
  IF unkOuter <> NIL then
    begin
      Result := CLASS_E_NOAGGREGATION;
      Exit;
    end;
  MyShellExtension := NIL;
  IF IsEqualIID(WantIID, IID_IUnknown) OR
     IsEqualIID(WantIID, IID_IShellExtInit) OR
     IsEqualIID(WantIID, IID_IShellCopyHook) OR
     IsEqualIID(WantIID, IID_IContextMenu) OR
     IsEqualIID(WantIID, CLSID_MyShellExtension) THEN
    MyShellExtension := IMyShellExtension.Create;
  Pointer(ReturnedObject) := MyShellExtension;
  IF MyShellExtension = NIL THEN
    begin
      Pointer(ReturnedObject) := NIL;
      Result := E_OUTOFMEMORY;
      Exit;
    end;
  Result := MyShellExtension.QueryInterface(WantIid, ReturnedObject);
  IF Failed(Result) THEN MyShellExtension.Free;
end;

function IMyClassFactory.LockServer(fLock: Bool): HResult;
begin
  IF fLock THEN Inc(LockCount)
  ELSE Dec(LockCount);
  Result := S_OK;
end;

initialization
  LockCount := 0;
  DllRefCount := 0;
end.
